/*
    AVFS: A Virtual File System Library
    Copyright (C) 1998  Miklos Szeredi (mszeredi@inf.bme.hu)

    This file can be distributed either under the GNU LGPL, or under
    the GNU GPL. See the file COPYING.LIB and COPYING. 
*/

#include "avfsint.h"
#include "virt.h"
#include "real.h"

#include <stdio.h>
#include <string.h>

static inline int maybe_local(const char *path)
{
  if(!__av_priv.vcwd_inited) __av_init_vcwd();
  if(!__av_priv.vcwd[0] || path == NULL || path[0] == DIR_SEP_CHAR) return 1;
  else return 0;
}

static inline int path_local(const char *path)
{
 return (strchr(path, VDEV_SEP_CHAR) == NULL) ? 1 : 0;
}

static inline int sure_local(ave *v, int res, const char *path)
{
  if(res != -1 || v->errn != ENOENT || path_local(path)) return 1;
  else return 0;
}


static inline finfo *check_fh(ave *v, int fh)
{
  if(!FD_OK(fh) || !ISVIRTUAL(fh)) {
    v->errn = EBADF;
    return NULL;
  }
  return &__av_priv.file_table[fh];
}

static inline void set_vcwd(const char *newvcwd)
{
  if(!__av_priv.vcwd_inited && !newvcwd[0]) return;

  strcpy(__av_priv.vcwd, newvcwd);
}

#define AV_REAL(name) __av_real ## name
#define AV_LOCAL(name) __av_local ## name

#define LOCAL1(name, rettyp, a1t) \
rettyp AV_LOCAL(name) (ave *v, a1t a1) \
{ rettyp res; errno = v->errn;          \
  res = AV_REAL(name) (a1);             \
  v->errn = errno; return res; }

#define LOCAL2(name, rettyp, a1t, a2t) \
rettyp AV_LOCAL(name) (ave *v, a1t a1, a2t a2) \
{ rettyp res; errno = v->errn;                  \
  res = AV_REAL(name) (a1, a2);                 \
  v->errn = errno; return res; }

#define LOCAL3(name, rettyp, a1t, a2t, a3t) \
rettyp AV_LOCAL(name) (ave *v, a1t a1, a2t a2, a3t a3) \
{ rettyp res; errno = v->errn;                          \
  res = AV_REAL(name) (a1, a2, a3);                     \
  v->errn = errno; return res; }

#define LOCAL4(name, rettyp, a1t, a2t, a3t, a4t) \
rettyp AV_LOCAL(name) (ave *v, a1t a1, a2t a2, a3t a3, a4t a4) \
{ rettyp res; errno = v->errn;                                  \
  res = AV_REAL(name) (a1, a2, a3, a4);                         \
  v->errn = errno; return res; }

#define LOCAL6(name, rettyp, a1t, a2t, a3t, a4t, a5t, a6t) \
rettyp AV_LOCAL(name) (ave *v,a1t a1,a2t a2,a3t a3,a4t a4,a5t a5,a6t a6) \
{ rettyp res; errno = v->errn;                                            \
  res = AV_REAL(name) (a1, a2, a3, a4, a5, a6);                           \
  v->errn = errno; return res; }
