/*  
    AVFS: A Virtual File System Library
    Copyright (C) 1998  Miklos Szeredi (mszeredi@inf.bme.hu)
    
    This file can be distributed either under the GNU LGPL, or under
    the GNU GPL. See the file COPYING.LIB and COPYING. 
*/

#include <dirent.h>
#include <utime.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>

extern int      __av_realopen       (const char *path, int flags, int mode);
extern int      __av_realopen64     (const char *path, int flags, int mode);
extern int      __av_realclose      (int fh);
extern ssize_t  __av_realread       (int fh, char *buf, size_t nbyte);
extern ssize_t  __av_realwrite      (int fh, const char *buf,size_t nbyte);
extern off_t    __av_reallseek      (int fh, off_t offset, int whence);
extern int      __av_realfstat      (int fh, struct stat *buf);
extern int      __av_realfstat64    (int fh, void *buf);
extern int      __av_realstat       (const char *path, struct stat *buf);
extern int      __av_realstat64     (const char *path, void *buf);
extern int      __av_reallstat      (const char *path, struct stat *buf);
extern int      __av_reallstat64    (const char *path, void *buf);
extern int      __av_realaccess     (const char *path, int amode);
extern int      __av_realreadlink   (const char *path, char *buf, size_t bsiz);
extern int      __av_realchdir      (const char *path);
extern int      __av_realfchdir     (int fh);
extern int      __av_realgetdents   (int fh, struct dirent *buf, size_t nbyte);
extern int      __av_realgetdents64 (int fh, void *buf, size_t nbyte);

extern int      __av_reallink       (const char *path, const char *newpath);
extern int      __av_realunlink     (const char *path);
extern int      __av_realchmod      (const char *path, mode_t mode);
extern int      __av_realchown      (const char *path, uid_t owner, gid_t grp);
extern int      __av_reallchown     (const char *path, uid_t owner, gid_t grp);
extern int      __av_realrmdir      (const char *path);
extern int      __av_realmkdir      (const char *path, mode_t mode);
extern int      __av_realmknod      (const char *path, mode_t mode, dev_t dev);
extern int      __av_realsymlink    (const char *path, const char *newpath);
extern int      __av_realrename     (const char *path, const char *newpath);
extern int      __av_realutime      (const char *path, struct utimbuf *buf);
extern int      __av_realutimes     (const char *path, struct timeval *buf);

extern int      __av_realfchmod     (int fh, mode_t mode);
extern int      __av_realfchown     (int fh, uid_t owner, gid_t grp);


extern int      __av_realexecve     (const char *filename, char *const argv[], 
				     char *const envp[]);


extern void    *__av_realmmap       (void *start, size_t len, int prot,
				     int flags, int fd, off_t offset);
extern int      __av_realacl        (const char *path, int cmd, 
				     int nent, void *aclbuf);
extern int      __av_realfacl       (int fd, int cmd, 
				     int nent, void *aclbuf);


extern DIR *          __av_realopendir   (const char *path);
extern int            __av_realclosedir  (DIR *dirp);
extern struct dirent *__av_realreaddir   (DIR *dirp);
extern off_t          __av_realtelldir   (DIR *dirp);
extern void           __av_realseekdir   (DIR *dirp, off_t loc);
extern void           __av_realrewinddir (DIR *dirp);

#ifdef HAVE_OPEN64
extern offset_t __av_reallseek64    (int fh, offset_t offset, int whence);
extern void    *__av_realmmap64     (void *start, size_t len, int prot,
				     int flags, int fd, offset_t offset);
#endif
